From e01d37b77b3e761a499850aa80de1c403f5dab86 Mon Sep 17 00:00:00 2001
From: amit <amitlangote09@gmail.com>
Date: Mon, 26 Jun 2017 15:23:34 +0900
Subject: [PATCH 3/3] Set pd_lower correctly in the SP-GiST index metapage

---
 src/backend/access/spgist/spgutils.c | 7 +++++++
 src/backend/access/spgist/spgxlog.c  | 8 +++-----
 2 files changed, 10 insertions(+), 5 deletions(-)

diff --git a/src/backend/access/spgist/spgutils.c b/src/backend/access/spgist/spgutils.c
index 22f64b0103..2f21f623da 100644
--- a/src/backend/access/spgist/spgutils.c
+++ b/src/backend/access/spgist/spgutils.c
@@ -534,6 +534,13 @@ SpGistInitMetapage(Page page)
 	/* initialize last-used-page cache to empty */
 	for (i = 0; i < SPGIST_CACHED_PAGES; i++)
 		metadata->lastUsedPages.cachedPage[i].blkno = InvalidBlockNumber;
+
+	/*
+	 * Set pd_lower just past the end of the metadata.  This is to log full
+	 * page image of metapage in xloginsert.c.
+	 */
+	((PageHeader) page)->pd_lower =
+		((char *) metadata + sizeof(SpGistMetaPageData)) - (char *) page;
 }
 
 /*
diff --git a/src/backend/access/spgist/spgxlog.c b/src/backend/access/spgist/spgxlog.c
index c440d21715..bf209416f2 100644
--- a/src/backend/access/spgist/spgxlog.c
+++ b/src/backend/access/spgist/spgxlog.c
@@ -1033,15 +1033,13 @@ void
 spg_mask(char *pagedata, BlockNumber blkno)
 {
 	Page		page = (Page) pagedata;
+	PageHeader	pagehdr = (PageHeader) page;
 
 	mask_page_lsn(page);
 
 	mask_page_hint_bits(page);
 
-	/*
-	 * Any SpGist page other than meta contains unused space which needs to be
-	 * masked.
-	 */
-	if (!SpGistPageIsMeta(page))
+	/* Mask the unused space, provided the page's pd_lower is set. */
+	if (pagehdr->pd_lower != 0)
 		mask_unused_space(page);
 }
-- 
2.11.0

